@@ -397,8 +397,11 @@ class GtkEditor(Editor):
 
     def __init__(self, config, loadfile = None):
         Editor.__init__(self, config, loadfile)
-        self.glade = glade.XML("%s/editor/magicor-editor.glade"
-                               %config.get("data_path", "data"))
+        gladeFile = "%s/editor/magicor-editor.glade"%config.get("data_path", "data")
+        try:
+            self.glade = glade.XML(gladeFile)
+        except RuntimeError, re:
+            raise RuntimeError("%s; glade file %s"%(re, gladeFile))
         self.glade.signal_autoconnect(self)
         if loadfile:
             self.setTitle(os.path.basename(loadfile))
