@@ -114,7 +114,7 @@ class Player(PhysicsSprite):
              (ImageFrame(self.IMAGE, 32, 48),
               SoundFrame("samples/push"),
               AnimationFrame(12, 2),
-              AnimationFrame(13, 4),
+              AnimationFrame(13, 2),
               AnimationFrame(12, 2),
               AnimationFrame(0, 0),
               AttributeFrame("pushing", False),
@@ -199,7 +199,7 @@ class Player(PhysicsSprite):
              (ImageFrame(self.IMAGE, 32, 48),
               SoundFrame("samples/push"),
               AnimationFrame(14, 2),
-              AnimationFrame(15, 4),
+              AnimationFrame(15, 2),
               AnimationFrame(14, 2),
               AnimationFrame(3, 0),
               AttributeFrame("pushing", False),
@@ -472,6 +472,10 @@ class Player(PhysicsSprite):
             self.setAnimation("happy")
             self._finished = True
             
+    def doAction(self,action):
+        if action=='die':
+            self.die()
+
     def physics(self):
         if self.dead:
             self.x += self.dieX
@@ -520,7 +524,8 @@ class Player(PhysicsSprite):
                                                              self.height,
                                                              self,
                                                              Tube)
-                            if tube or not self.blockedRightAbove():
+                            if ((tube and tube.direction == "left")
+                                or not self.blockedRightAbove()):
                                 self.jumping = True
                                 if tube:
                                     self.setAnimation("jump-tube-right")
@@ -528,7 +533,8 @@ class Player(PhysicsSprite):
                                 else:
                                     self.setAnimation("jump-right")
                                 self.moving = 0
-                        elif block and not block.blockedRight():
+                        elif (block and not block.blockedRight() and
+                            block.y+4<self.y+32): #block.top+4<self.bottom
                             self.pushing = True
                             block.push(1)
                             self.setAnimation("push-right")
@@ -565,7 +571,8 @@ class Player(PhysicsSprite):
                                                              self.height,
                                                              self,
                                                              Tube)
-                            if tube or not self.blockedLeftAbove():
+                            if ((tube and tube.direction == "right")
+                                or not self.blockedLeftAbove()):
                                 self.jumping = True
                                 if tube:
                                     self.setAnimation("jump-tube-left")
@@ -573,7 +580,8 @@ class Player(PhysicsSprite):
                                 else:
                                     self.setAnimation("jump-left")
                                 self.moving = 0
-                        elif block and not block.blockedLeft():
+                        elif ( block and not block.blockedLeft() and
+                               block.y+4<self.y+32): #block.top+4<self.bottom
                             self.pushing = True
                             block.push(-1)
                             self.setAnimation("push-left")
@@ -617,3 +625,21 @@ class Player(PhysicsSprite):
                      (self._frameIndex * self.width,
                       0, self.width, self.height + 16)
                      )
+
+    def hack_info_vel(self):
+        if self.vertical:
+            self.veldir[1]=self.vertical/abs(self.vertical)
+            self.fasteness = 4
+        else: self.veldir[1]=0
+        if ( self.jumping
+             or self.freezing
+             or self.pushing ):
+            self.veldir[0]=0
+            self.fasteness = 0
+        elif not self.moving and self.isDone:
+            self.veldir[0] = 0
+            self.fasteness = 0
+        else:
+            self.veldir[0] = self.direction
+            self.fasteness = 4
+
