@@ -8,15 +8,16 @@ Copyright 2006  Peter Gebauer. Licensed as Public Domain.
 import pygame, time, os, warnings
 from pygame.mixer import music
 
-from magicor import Text
+from magicor import Text, set_group
 from magicor.resources import ResourceNotFound
 from magicor.states import MenuState, BaseState, ErrorState
 from magicor.states.options import MainOptionsState
 from magicor.sprites import AnimationGroup
 from magicor.sprites.blocks import BlocksGroup, Ice, NormalIce
-from magicor.sprites.world import Lava, Tube
+from magicor.sprites.world import Lava, Tube, Trapola
 from magicor.sprites.fires import Fire
 from magicor.sprites.player import Player
+from magicor.sprites.stones import Ball
 from magicor.sprites.decorations import Decoration
 from magicor.sprites.enemies import (WalkingEnemy,
                                      ClimbingEnemy,
@@ -73,12 +74,19 @@ class PlayState(BaseState):
             self.resources[self.config["default_tile"]])
         self.level = level
         self.blocks = BlocksGroup()
+        set_group( 'blocks', self.blocks )
         self.decorations = AnimationGroup()
         self.fires = AnimationGroup()
+        set_group( 'fires', self.fires )
         self.world = AnimationGroup()
+        set_group( 'world', self.world )
         self.lights = AnimationGroup()
         self.enemies = AnimationGroup()
+        set_group( 'enemies', self.enemies )
         self.players = AnimationGroup()
+        set_group( 'players', self.players )
+        self.stones = AnimationGroup()
+        set_group( 'stones', self.stones )
         self.hudSprites = AnimationGroup()
         self.starting = 17
         self.startTime = time.time()
@@ -151,12 +159,29 @@ class PlayState(BaseState):
                 elif arg == "connect-right":
                     ice.addConnections(1)
             elif name == "lava":
+                ##->@@spitting lava modifications
+                bSpiting = False
+                t=None
+                if arg is None:
+                    bDormant=False
+                else:
+                    spl=arg.split(' ')
+                    if  spl[0]=='dormant':
+                        bDormant=True
+                    else:
+                        bSpiting=True
+                        bDormant=bool(spl[0]=='spit_dorm')
+                        if len(spl)<2: t=16
+                        else: t = int(spl[3])
                 self.world.add(Lava(x * 32, y * 32,
                                     self.blocks,
                                     self.players,
                                     self.fires,
                                     self.world,
-                                    bool(arg == "dormant")))
+                                    bDormant,
+                                    bSpiting,
+                                    t))
+                ##<-@@spitting lava modifications
             elif name == "fire":
                 self.fires.add(Fire(x * 32, y * 32,
                                     self.level,
@@ -233,6 +258,18 @@ class PlayState(BaseState):
                                                  self.players,
                                                  self.fires,
                                                  self.world))
+            elif name == "ball":
+                 bReadOk, direction, fasteness, dvy = Ball.parse(arg)
+                 if bReadOk:
+                     self.stones.add(Ball(self.level, x*32, y*32,direction,fasteness,dvy))
+
+            elif name == "trapola":
+                self.world.add(Trapola(x*32,y*32-1))
+                if not self.level[x,y]:
+                    #TODO: put a suitable tile if empty - at this time level
+                    #designers should put a tile behind the trapola
+                    self.level[x, y] = "!"
+
             elif name == "direction":
                 self.hudSprites.add(Direction(x * 32, y * 32, arg))
         if not self.player:
@@ -352,6 +389,17 @@ class PlayState(BaseState):
                 v = 16 + self.ending
                 if self.ending == 0:
                     if self.player._finished:
+			this_time = int(time.time() - self.startTime)
+			if self.config.has_key("time_"+self.level.title):
+				best_time = min(self.config.getInt("time_"+self.level.title), this_time)
+			else:
+				best_time = this_time
+			print "best time for "+self.level.title+" is "+str(best_time)
+			self.config["time_"+self.level.title] = best_time
+                	fn = "%s/magicor.conf"%self.config.get("user_path", "~/.magicor")
+                	self.config.saveFile(fn)
+
+
                         self._next = self.previous(self.config,
                                                    self.data,
                                                    self.screen)
@@ -366,15 +414,16 @@ class PlayState(BaseState):
             self.renderSurface.blit(self.levelSurface, (0, 0))
             self.fires.animate()
             self.fires.draw(self.renderSurface)
-            self.players.draw(self.renderSurface)
             self.enemies.draw(self.renderSurface)
             self.world.animate()
-            self.world.draw(self.renderSurface)
-            self.decorations.animate()
-            self.decorations.draw(self.renderSurface)
             self.lights.update()
             self.lights.draw(self.renderSurface)
             self.blocks.draw(self.renderSurface)
+            self.players.draw(self.renderSurface)
+            self.world.draw(self.renderSurface)
+            self.stones.draw(self.renderSurface)
+            self.decorations.animate()
+            self.decorations.draw(self.renderSurface)
             self.hudSprites.update()
             self.hudSprites.draw(self.renderSurface)
             self.screen.blit(self.renderSurface, (80, 16))
@@ -399,15 +448,17 @@ class PlayState(BaseState):
             self.fires.update()
             self.fires.draw(self.renderSurface)
             self.players.update()
-            self.players.draw(self.renderSurface)
             self.enemies.update()
             self.enemies.draw(self.renderSurface)
             self.world.update()
+            self.blocks.update()
+            self.stones.update()
+            self.blocks.draw(self.renderSurface)
+            self.players.draw(self.renderSurface)
             self.world.draw(self.renderSurface)
             self.decorations.update()
             self.decorations.draw(self.renderSurface)
-            self.blocks.update()
-            self.blocks.draw(self.renderSurface)
+            self.stones.draw(self.renderSurface)
             if self.config.get("eyecandy"):
                 self.lights.update()
                 self.lights.draw(self.renderSurface)
